#include <stdio.h>
#include "cv.h"
#include "cxcore.h"
#include "highgui.h"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

double scaling = 1.0;
double w[3 * 3] = { 0.0,  -1.0,  0.0, 
                    -1.0,  5.0,  -1.0,
                    0.0,  -1.0,  0.0 }; 

/*
#define RESULT_SIGNED
double scaling = 1.0;
double w[3 * 3] = { 0.0,  1.0,  0.0, 
                    1.0, -4.0,  1.0,
                    0.0,  1.0,  0.0 }; 

double scaling = 1.0;
double w[3 * 3] = { 0.0,  -1.0,  0.0, 
                    -1.0,  5.0,  -1.0,
                    0.0,  -1.0,  0.0 }; 

double scaling = 1.0 / 8.0;
double w[3 * 3] = { 0.0,  1.0,  0.0, 
                    1.0,  4.0,  1.0,
                    0.0,  1.0,  0.0 }; 

#define RESULT_SIGNED
double scaling = 1.0;
double w[3 * 3] = { -1.0,  0.0,  1.0, 
                    -2.0,  0.0,  2.0,
                    -1.0,  0.0,  1.0 }; 
 */


#ifdef RESULT_SIGNED
#define RESULT_TYPE IPL_DEPTH_8S
#define CLIP_MAX 127
#define CLIP_MIN -128
#else
#define RESULT_TYPE IPL_DEPTH_8U
#define CLIP_MAX 255
#define CLIP_MIN 0
#endif

uchar clipvalue(double v)
{
    int vi = (int)v;
    if (vi >= CLIP_MAX) {
        return CLIP_MAX;
    } else if (vi <= CLIP_MIN) {
        return CLIP_MIN;
    } else {
        return (uchar)vi;
    }
}

uchar
pval(IplImage *img, int x, int y)
{
    int h = img->height;
    int w = img->width;

    if (x < 0 || x >= w || y < 0 || y >= h) {
	    return 0;
    }

    return PIXVAL(img, x, y);
}

void
filter3x3(IplImage *img, IplImage *result)
{
    int i, j, m, n;

    for (j = 0; j < img->height; j++) {
        for (i = 0; i < img->width; i++) {
            double sum = 0.0;
            for (n = 0; n < 3; n++) {
                for (m = 0; m < 3; m++) {
                    sum += pval(img, i - 1 + m, j - 1 + n)
                        * w[m + 3 * n];
                }
            }
            PIXVAL(result, i, j) = clipvalue(scaling * sum);
        }
    }
}

int
main()
{
    IplImage *frame, *buf, *img, *result;
    CvCapture *capture = cvCaptureFromCAM(0);
    if (capture == NULL) {
        fprintf(stderr, "cannot capture\n");
        return 1;
    }

    frame = cvQueryFrame(capture);
    buf = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 3);  // color
    img = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);  // mono
    result = cvCreateImage(cvGetSize(frame), RESULT_TYPE, 1);  // mono

    cvNamedWindow("result", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("original", CV_WINDOW_AUTOSIZE);

    while (1) {
        frame = cvQueryFrame(capture); /* cannot be modified */

        cvFlip(frame, buf, NULL);
        cvCvtColor(buf, img, CV_BGR2GRAY);

        filter3x3(img, result);
        filter3x3(result, img);
        filter3x3(img, result);

		cvCvtColor(buf, img, CV_BGR2GRAY);
        cvShowImage("original", img);
        cvShowImage("result", result);

        if (cvWaitKey(30) > 0) {
            break;
        }
    }

    return 0;
}
