#include <stdio.h>
#include "cv.h"
#include "cxcore.h"
#include "highgui.h"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

void
dilation(IplImage *src, IplImage *dst)
{
    int i, j;
    IplImage *tmp = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
    for (j = 1; j < src->height - 1; j++) {
        for (i = 1; i < src->width - 1; i++) {
            PIXVAL(tmp, i, j) 
                = 255 * (PIXVAL(src, i, j) 
                         || PIXVAL(src, i - 1, j)
                         || PIXVAL(src, i + 1, j)
                         || PIXVAL(src, i, j - 1)
                         || PIXVAL(src, i, j + 1));
        }
    }
    cvCopy(tmp, dst, NULL);
    cvReleaseImage(&tmp);
}

void
erosion(IplImage *src, IplImage *dst)
{
    int i, j;
    IplImage *tmp = cvCreateImage(cvGetSize(src), IPL_DEPTH_8U, 1);
    for (j = 1; j < src->height - 1; j++) {
        for (i = 1; i < src->width - 1; i++) {
            PIXVAL(tmp, i, j) 
                = 255 * (PIXVAL(src, i, j) 
                         && PIXVAL(src, i - 1, j)
                         && PIXVAL(src, i + 1, j)
                         && PIXVAL(src, i, j - 1)
                         && PIXVAL(src, i, j + 1));
        }
    }
    cvCopy(tmp, dst, NULL);
    cvReleaseImage(&tmp);
}

int main()
{
    int i, j;

    CvCapture *capture = cvCaptureFromCAM(0);

    IplImage *frame, *buf, *img, *bin, *result;

    if (capture == NULL) {
        fprintf(stderr, "cannot capture\n");
        return 1;
    }

    frame = cvQueryFrame(capture);
    buf = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 3);  // color
    img = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    bin = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    result = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);

    cvNamedWindow("input", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("binary", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("result", CV_WINDOW_AUTOSIZE);

    while (1) {
        frame = cvQueryFrame(capture); /* cannot be modified */

        cvFlip(frame, buf, NULL);
        cvCvtColor(buf, img, CV_BGR2GRAY);

        for (j = 0; j < img->height; j++) {
            for (i = 0; i < img->width; i++) {
                PIXVAL(bin, i, j) = (PIXVAL(img, i, j) > 128)? 255: 0;
            }
        }
        dilation(bin, result);
        dilation(result, result);
        dilation(result, result);
		dilation(result, result);
		
		erosion(result, result);
        erosion(result, result);
        erosion(result, result);
        erosion(result, result);

        cvShowImage("input", img);
        cvShowImage("binary", bin);
        cvShowImage("result", result);

        if (cvWaitKey(30) > 0) {
            break;
        }
    }

    return 0;
}
